(function(once) {

  const deprecatedMessage = 'The selected component is deprecated and will be removed. Evaluate migrating to a stable component.';

  window.optionsFilter = {
    states: [],

    initIndex: function(index = 0) {
      optionsFilter.states[index] = {};
    },

    setContainer: function(container, index = 0) {
      optionsFilter.states[index].container = container;
      optionsFilter.states[index].components = Array.from(container.querySelectorAll('.js-form-type-radio'));
      var deprecationCheckbox = container.querySelector('input.deprecation-checkbox');
      optionsFilter.states[index].deprecatedFilterValue = true;
      if (deprecationCheckbox !== null) {
        optionsFilter.states[index].deprecatedFilterValue = container.querySelector(
          'input.deprecation-checkbox').checked;
      }
      optionsFilter.states[index].infoLayer = container.querySelector('.currently-selected');
      optionsFilter.states[index].infoLayer.hidden = true;
      optionsFilter.states[index].warningLayer = container.querySelector('.warning');
      optionsFilter.states[index].warningLayer.hidden = true;

      optionsFilter.setDefaultComponentInDOM(index);
    },

    getContainer: function(index = 0) {
      return optionsFilter.states[index].container;
    },

    getInfoLayer: function(index = 0) {
      return optionsFilter.states[index].infoLayer;
    },

    setSearchboxValue: function(value, index = 0) {
      optionsFilter.states[index].searchboxValue = value;
    },

    getSearchboxValue: function(index = 0) {
      return optionsFilter.states[index].searchboxValue;
    },

    setDeprecatedFilterValue: function(value, index = 0) {
      optionsFilter.states[index].deprecatedFilterValue = value;
    },

    getDeprecatedFilterValue: function(index = 0) {
      return optionsFilter.states[index].deprecatedFilterValue;
    },

    setSelectedComponent: function(selectedComponent, index = 0) {
      optionsFilter.states[index].selectedComponent = selectedComponent;
    },

    getSelectedComponent: function(index = 0) {
      return optionsFilter.states[index].selectedComponent;
    },

    setDefaultComponentInDOM: function(index = 0) {
      const container = optionsFilter.getContainer(index);
      const selectedComponent = container.querySelector('input[type="radio"]:checked');
      if (selectedComponent === null) {
        return null;
      }
      optionsFilter.selectComponent(selectedComponent, index);
    },

    setSearchboxValueInDOM: function(value, index = 0) {
      const container = optionsFilter.getContainer(index);
      const searchBox = container.querySelector('input.search-box');
      searchBox.value = value;
      optionsFilter.setSearchboxValue(value);
    },

    selectComponent: function(selectedComponent, index = 0) {
      var componentWrapper = selectedComponent.closest('.js-form-type-radio');
      optionsFilter.setSelectedComponent(componentWrapper, index);
      optionsFilter.states[index].components.map(function(element) {
        element.classList.remove('form-type--radio__selected');
      });
      componentWrapper.classList.add('form-type--radio__selected');
      optionsFilter.showWarningMessageIfNeeded(index);
      optionsFilter.setSearchboxValueInDOM(componentWrapper.querySelector('.radio-details--machine-name').innerText);
    },

    showAllComponents: function(index = 0) {
      optionsFilter.states[index].components.map(function(element) {
        element.hidden = false;
      });
    },

    showSelectedComponent: function(index = 0) {
      optionsFilter.getSelectedComponent(index).hidden = false;
    },

    hideAllComponents: function(index = 0) {
      optionsFilter.states[index].components.map(function(element) {
        element.hidden = true;
      });
    },

    hideAllComponentsBut: function(componentsToShow, index = 0) {
      optionsFilter.hideAllComponents(index);
      for (const matchingElement of componentsToShow) {
        matchingElement.parentElement.parentElement.hidden = false;
      }
    },

    filterComponentsSearched: function(index = 0) {
      const container = optionsFilter.getContainer(index);
      const searchboxValue = optionsFilter.getSearchboxValue(index);
      const matchingComponents = Array.from(
        container.querySelectorAll('.radio-details--search')).
        filter(e => e.innerText.search(searchboxValue) !== -1);
      optionsFilter.hideAllComponentsBut(matchingComponents, index);
    },

    isComponentDeprecated: function(component) {
      return component.innerText.search('deprecated') >= 0;
    },

    getDeprecatedComponents: function(index = 0) {
      return optionsFilter.states[index].components.filter(function(component) {
        return optionsFilter.isComponentDeprecated(component);
      });
    },

    filterComponentsDeprecated: function(index = 0) {
      var showDeprecated = optionsFilter.getDeprecatedFilterValue(index);
      optionsFilter.getDeprecatedComponents(index).map(function(component) {
        if (!showDeprecated) {
          component.hidden = true;
        }
      });
    },

    showWarningMessageIfNeeded: function(index = 0) {
      var selectedComponent = optionsFilter.getSelectedComponent(index);
      optionsFilter.states[index].warningLayer.hidden = true;
      optionsFilter.states[index].warningLayer.innerText = '';
      if (optionsFilter.isComponentDeprecated(selectedComponent)) {
        optionsFilter.states[index].warningLayer.hidden = false;
        optionsFilter.states[index].warningLayer.innerText = deprecatedMessage;
      }
    },

    refreshComponents: function(index = 0) {
      optionsFilter.showAllComponents(index);
      optionsFilter.filterComponentsSearched(index);
      optionsFilter.filterComponentsDeprecated(index);
      optionsFilter.showSelectedComponent(index);
    },

    renderSelectedComponent: function(index = 0) {
      var info = optionsFilter.getInfoLayer(index);
      var selectedComponent = optionsFilter.getSelectedComponent(index);

      var id = selectedComponent.querySelector('input[type="radio"]').value;
      var name = selectedComponent.querySelector('.radio-details--human-name').innerText;
      var description = selectedComponent.querySelector('.radio-details--description').innerText;
      var status = selectedComponent.querySelector('.radio-details--status').innerText;
      var group = selectedComponent.querySelector('.radio-details--group').innerText;
      var documentation = selectedComponent.querySelector('.radio-details--documentation').innerHTML;
      const imgElement = selectedComponent.querySelector('.radio-details--image');
      var img = imgElement ? imgElement.outerHTML : '';
      info.innerHTML = Drupal.theme(
        'currentlySelectedComponent',
        id,
        name,
        description,
        status,
        group,
        documentation,
        img
      );
      info.hidden = false;
    },
  };

  const subscribeSearchboxToChanges = function(index = 0) {
    const container = optionsFilter.getContainer(index);
    const searchBox = container.querySelector('input.search-box');
    if (searchBox === null) {
      return;
    }
    searchBox.addEventListener('input', function(event) {
      optionsFilter.setSearchboxValue(event.target.value, index);
      optionsFilter.refreshComponents(index);
    });
  };

  const includeResetButton = function(index) {
    const container = optionsFilter.getContainer(index);
    const searchBox = container.querySelector('input.search-box');
    if (searchBox === null) {
      return;
    }
    var resetButton = document.createElement('button');
    resetButton.classList.add('reset-search-box');
    resetButton.innerText = '⨯';
    resetButton.title = 'Reset filter';
    resetButton.addEventListener('click', function(event) {
      event.preventDefault();
      container.querySelector('input.search-box').value = '';
      optionsFilter.setSearchboxValue('', index);
      optionsFilter.refreshComponents(index);
      event.target.blur();
    });

    searchBox.parentElement.insertBefore(resetButton, searchBox.nextSibling);
  };

  const subscribeDeprecatedSearchboxToChanges = function(index = 0) {
    const container = optionsFilter.getContainer(index);
    const deprecationCheckbox = container.querySelector('input.deprecation-checkbox');
    if (deprecationCheckbox === null) {
      return;
    }
    deprecationCheckbox.addEventListener('change', function(event) {
      optionsFilter.setDeprecatedFilterValue(event.target.checked, index);
      optionsFilter.refreshComponents(index);
    });
  };

  const subscribeRadiosToChanges = function(index = 0) {
    const container = optionsFilter.getContainer(index);
    var radios = once('radio-change-subscribed', 'input[type="radio"]', container);
    radios.map(function(radio) {
      radio.addEventListener('change', function(event) {
        optionsFilter.selectComponent(event.target, index);
        optionsFilter.refreshComponents(index);
        optionsFilter.renderSelectedComponent(index);
      });
    });
  };

  Drupal.behaviors.optionsFilter = {
    attach: (context, settings) => {
      once('options-filter', '.component--selector', context).map(function(container, index) {
        optionsFilter.initIndex(index);
        optionsFilter.setContainer(container, index);
        subscribeSearchboxToChanges(index);
        subscribeDeprecatedSearchboxToChanges(index);
        subscribeRadiosToChanges(index);
        includeResetButton(index);
        optionsFilter.refreshComponents(index);
        if (optionsFilter.getSelectedComponent(index) !== undefined) {
          optionsFilter.renderSelectedComponent(index);
        }
      });
    },
  };

  Drupal.theme.currentlySelectedComponent = (
    id,
    name,
    description,
    status,
    group,
    documentation,
    img
  ) => `
    <summary>${Drupal.t('ℹ️ More information about <em>@name</em>',
    {'@name': name})}</summary>
    <p>${description}</p>
    <div class='image-table--wrapper'>
      <table>
        <tr><th>${Drupal.t('ID')}</th><td><code>${id}</code></td></tr>
        <tr><th>${Drupal.t('Status')}</th><td>${status}</td></tr>
        <tr><th>${Drupal.t('Group')}</th><td>${group}</td></tr>
      </table>
      <div class='currently-selected--image--wrapper${img
    ? ''
    : ' currently-selected--image--wrapper__empty'}'>
        ${img ? img : ''}
      </div>
    </div>
    <div class='currently-selected--readme'>${documentation}</div>
  `;
}(once));
